unit AboutFrm;

{************************************************************************
*                                                                       *
*                       Base Calculator About Box                       *
*                Copyright (C) 1993-2000, John Zaitseff                 *
*                                                                       *
************************************************************************}

{ Author:    John Zaitseff <J.Zaitseff@zap.org.au>
  Date:      1st April, 2000.
  Version:   2.1

  This file provides the user interface code to create a simple About
  dialog box.  Most of the "code" is actually written by Delphi in the
  file AboutFrm.dfm.  Note that the About dialog box has TWO images, one
  on top of the other.  BCalcImage256 is displayed on 256-colour (or
  higher) displays; BCalcImage16 is displayed on 16-colour displays.

  This file is supplied as part of an application under the GNU General
  Public License.  Hence this file also falls under the same license.
}


{$IFNDEF WIN32}
Error: 'Compile under Borland Delphi for Win32 only'
{$ENDIF}


interface


uses
  Windows, ShellAPI, Classes, Graphics, Controls, Forms, ExtCtrls, StdCtrls;


type
  TAboutWin = class(TForm)
    Panel          : TPanel;
    BCalcImage256  : TImage;
    BCalcImage16   : TImage;
    ProductName    : TLabel;
    VersionStr     : TLabel;
    Copyright      : TLabel;
    LatestInfoText : TLabel;
    WebAddress     : TLabel;
    Bevel1         : TBevel;
    Comments1      : TLabel;
    Comments2      : TLabel;
    OKBtn          : TButton;
    LicenseBtn     : TButton;

    procedure FormShow (Sender : TObject);
    procedure LicenseBtnClick (Sender : TObject);
    procedure WebAddressClick (Sender : TObject);
  end;


var
  AboutWin : TAboutWin;


implementation


uses
  StringRes;


{$R *.DFM}


{***********************************************************************}
{ Show the About dialog box, making it centred above its parent form.
  This procedure also makes one of the images (BCalcImage256 or BCalcImage16)
  visible, depending on if the display can support more than 16 colours }
procedure TAboutWin.FormShow (Sender : TObject);

var
  ParentForm : TForm;
  DC         : HDC;
  NumColours : integer;
  UseBM16    : boolean;

begin
  if Owner is TApplication then
    ParentForm := TApplication(Owner).MainForm
  else if Owner is TForm then
    ParentForm := TForm(Owner)
  else
    ParentForm := nil;

  { Centre the form above its parent }
  if Assigned(ParentForm) then
    begin
      Left := (ParentForm.Left + ParentForm.Width div 2) - Width div 2;
      Top := (ParentForm.Top + ParentForm.Height div 2) - Height div 2
    end
  else
    Position := poScreenCenter;

  { Check if device has more than 16 fixed colours }
  DC := GetDC(0);
  NumColours := GetDeviceCaps(DC, NUMCOLORS);
  ReleaseDC(0,DC);
  UseBM16 := (NumColours > 0) and (NumColours <= 16);

  { Display just one of the images }
  BCalcImage256.Visible := not UseBM16;
  BCalcImage16.Visible := UseBM16;

  { Set the Web address to the correct value }
  WebAddress.Caption := HomePageURL
end;


{***********************************************************************}
{ Display the program's license from the Help file, using the AboutWin's
  HelpContext property }
procedure TAboutWin.LicenseBtnClick (Sender : TObject);

begin
  Application.HelpContext(HelpContext)
end;


{***********************************************************************}
{ Use a Web browser to display the URL contained in HomePageURL.  This
  function uses ShellExecute and assumes that a Web browser has been
  correctly installed }
procedure TAboutWin.WebAddressClick (Sender : TObject);

begin
  ShellExecute(0, 'open', PChar(HomePageURL), nil, nil, SW_SHOWNORMAL)
end;

end.

