unit MainFrm;

{************************************************************************
*                                                                       *
*                    Base Calculator User Interface                     *
*                Copyright (C) 1993-2000, John Zaitseff                 *
*                                                                       *
************************************************************************}

{ Author:    John Zaitseff <J.Zaitseff@zap.org.au>
  Date:      1st April, 2000.
  Version:   2.1

  This file provides the user interface for the Base Calculator.  It
  contains all of the code for the main window, TMainWin.

  Note that Borland Delphi 4 or later is required to compile this unit.

  Please note that the Tag property of buttons is used to determine the
  parameter for TCalculator.HandleKey, and is the ordinal of TCalcKey
  for each button.  For example, the "SHL" button, BtnSHL, should call
  TCalculator.HandleKey with a parameter of ckSHL.  Hence, BtnSHL.Tag
  contains Ord(ckSHL), ie, 29.

  This file is supplied as part of an application under the GNU General
  Public License.  Hence this file also falls under the same license.
}


{$IFNDEF WIN32}
Error: 'Compile under Borland Delphi for Win32 only'
{$ENDIF}


interface


uses
  Windows, ShellAPI, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, Menus, ExtCtrls, StdCtrls, ComCtrls, ToolWin, ClipBrd,
  ActnList, ImgList, Calc, BCConfig, BCDefs, BCDisp, BCFonts, Tips;


const
  BeepType      = MB_IconExclamation;  { Speaker sound to produce on an error }
  HelpAutoPopup = 500;                 { Help context IDs for which to generate }
                                       { HELP_CONTEXTPOPUP; see FormHelp        }
  HelpNoPopup   = 5000;                { Help context IDs for which to suppress }
                                       { HELP_CONTEXT; see FormHelp             }


type
  TMainWin = class(TForm)
      DisplayPanel    : TPanel;
      BCalcDisplay    : TBCalcDisplay;
      Btn0            : TButton;
      Btn1            : TButton;
      Btn2            : TButton;
      Btn3            : TButton;
      Btn4            : TButton;
      Btn5            : TButton;
      Btn6            : TButton;
      Btn7            : TButton;
      Btn8            : TButton;
      Btn9            : TButton;
      BtnA            : TButton;
      BtnB            : TButton;
      BtnC            : TButton;
      BtnD            : TButton;
      BtnE            : TButton;
      BtnF            : TButton;
      BtnMultiply     : TButton;
      BtnMinus        : TButton;
      BtnPlus         : TButton;
      BtnDivide       : TButton;
      BtnMod          : TButton;
      BtnMPlus        : TButton;
      BtnLParen       : TButton;
      BtnRParen       : TButton;
      BtnMMinus       : TButton;
      BtnEquals       : TButton;
      BtnClear        : TButton;
      BtnLeft         : TButton;
      BtnSwapXY       : TButton;
      BtnSwapXM       : TButton;
      BtnMIn          : TButton;
      BtnMR           : TButton;
      BtnNot          : TButton;
      BtnAnd          : TButton;
      BtnOr           : TButton;
      BtnXor          : TButton;
      BtnSHL          : TButton;
      BtnSHR          : TButton;
      BtnNeg          : TButton;
      BtnNand         : TButton;
      BtnNor          : TButton;
      BtnEqv          : TButton;
      BtnROL          : TButton;
      BtnROR          : TButton;
      BtnsBevel       : TBevel;
      BCalcImageList  : TImageList;
      BCalcActionList : TActionList;
      FileOptions     : TAction;
      FileExit        : TAction;
      EditCopy        : TAction;
      EditPaste       : TAction;
      ModeDec         : TAction;
      ModeHex         : TAction;
      ModeBin         : TAction;
      ModeOct         : TAction;
      ModeBit8        : TAction;
      ModeBit16       : TAction;
      ModeBit32       : TAction;
      ModeBit64       : TAction;
      ModeBit96       : TAction;
      ModeBit128      : TAction;
      ModeSigned      : TAction;
      HelpTopics      : TAction;
      HelpTipOfDay    : TAction;
      HelpWeb         : TAction;
      HelpAbout       : TAction;
      MainMenu        : TMainMenu;
      FileMI          : TMenuItem;
      OptionsMI       : TMenuItem;
      Sep1MI          : TMenuItem;
      ExitMI          : TMenuItem;
      EditMI          : TMenuItem;
      CopyMI          : TMenuItem;
      PasteMI         : TMenuItem;
      Sep2MI          : TMenuItem;
      ModeMI          : TMenuItem;
      DecMI           : TMenuItem;
      HexMI           : TMenuItem;
      BinMI           : TMenuItem;
      OctMI           : TMenuItem;
      Sep3MI          : TMenuItem;
      Bit8MI          : TMenuItem;
      Bit16MI         : TMenuItem;
      Bit32MI         : TMenuItem;
      Bit64MI         : TMenuItem;
      Bit96MI         : TMenuItem;
      Bit128MI        : TMenuItem;
      Sep4MI          : TMenuItem;
      SignedMI        : TMenuItem;
      HelpMI          : TMenuItem;
      ContentsMI      : TMenuItem;
      Sep5MI          : TMenuItem;
      TipOfDayMI      : TMenuItem;
      WebSiteMI       : TMenuItem;
      Sep6MI          : TMenuItem;
      AboutMI         : TMenuItem;
      PopupEditMenu   : TPopupMenu;
      WhatPMI         : TMenuItem;
      Sep1PMI         : TMenuItem;
      CopyPMI         : TMenuItem;
      PastePMI        : TMenuItem;
      PopupWhatMenu   : TPopupMenu;
      What2PMI        : TMenuItem;
      ShortcutsTB     : TToolBar;
      DecTBtn         : TToolButton;
      HexTBtn         : TToolButton;
      BinTBtn         : TToolButton;
      OctTBtn         : TToolButton;
      Sep1TB          : TToolButton;
      Bit8TBtn        : TToolButton;
      Bit16TBtn       : TToolButton;
      Bit32TBtn       : TToolButton;
      Bit64TBtn       : TToolButton;
      Bit96TBtn       : TToolButton;
      Bit128TBtn      : TToolButton;
      Sep2TB          : TToolButton;
      SignedTBtn      : TToolButton;
      Sep3TB          : TToolButton;
      CopyTBtn        : TToolButton;
      PasteTBtn       : TToolButton;
      Sep4TB          : TToolButton;
      WebSiteTBtn     : TToolButton;
      HelpTBtn        : TToolButton;
      TipsDialog      : TTipsDialog;
      TipOfDayTimer   : TTimer;

      procedure FormCreate (Sender : TObject);
      procedure FormClose (Sender : TObject; var Action : TCloseAction);
      procedure FormDestroy (Sender : TObject);

      procedure CalcKeyBtnClick (Sender : TObject);
      procedure FormKeyDown (Sender : TObject; var Key : word; Shift : TShiftState);
      procedure FormShortCut (var Msg : TWMKey; var Handled : boolean);
      procedure FormKeyPress (Sender : TObject; var Key : char);

      procedure FileOptionsExecute (Sender : TObject);
      procedure FileExitExecute (Sender : TObject);
      procedure EditCopyExecute (Sender : TObject);
      procedure EditPasteExecute (Sender : TObject);
      procedure EditUpdateActions (Sender : TObject);
      procedure ModeBaseExecute (Sender : TObject);
      procedure ModeBitExecute (Sender : TObject);
      procedure ModeSignedExecute (Sender : TObject);
      procedure ModeUpdateActions (Sender : TObject);
      procedure HelpTopicsExecute (Sender : TObject);
      procedure HelpTipOfDayExecute (Sender : TObject);
      procedure HelpWebExecute (Sender : TObject);
      procedure HelpAboutExecute (Sender : TObject);
      procedure EditWhatExecute (Sender : TObject);
      procedure HelpWhatExecute (Sender : TObject);
      function FormHelp (Command : word; Data : integer;
                         var CallHelp : boolean) : boolean;


    private
      { Private declarations }
      FCalculator : TCalculator;
      FConfig     : TBCConfig;

      FTipsLoaded : boolean;       { True if tips loaded from disk }

      procedure WMDisplayChange (var Message : TWMDisplayChange); message WM_DISPLAYCHANGE;

      procedure LoadFonts (FontResID : TResID);
      procedure LoadTips;
      procedure UpdateDisplay;
      procedure UpdateButtons;

      function CharToButton (C : char) : TButton;

    public
      { Public declarations }
      property Calculator : TCalculator read FCalculator;
      property Config     : TBCConfig   read FConfig;
    end;


var
  MainWin : TMainWin;


implementation


uses
  AboutFrm, OptionsFrm, StringRes, VerRes;


{$R *.DFM}


{***********************************************************************}
{ Create the main Base Calculator window and the associated state machine,
  reading the user's preferences from the system registry }
procedure TMainWin.FormCreate (Sender : TObject);

var
  AppVerRes : TVerRes;
  StatFont  : TPicture;

begin
  { Read Base Calculator's version number, as stored in the VERSIONINFO
    structure of the executable.  Use the ProductVersion key as the
    version identifier to read the user's settings from the registry }
  AppVerRes := TVerRes.Create;
  try
    FConfig := TBCConfig.Create(RegPathPrefix + AppVerRes.ProductVersion)
  finally
    AppVerRes.Free
  end;

  { Create the calculator state machine }
  FCalculator := TCalculator.Create(Config.MaxParenLevel);

  { Load the status font }
  StatFont := TPicture.Create;
  try
    StatFont.Bitmap.LoadFromResourceID(HInstance, Config.StatusFontResID);
    BCalcDisplay.StatusFont := StatFont
  finally
    StatFont.Free
  end;

  { Load the four user fonts }
  LoadFonts(Config.FontResID);

  { Apply the user's preferences }

  with Calculator do
    begin
      OpPrecStyle := Config.OpPrecStyle;
      Mode := Config.CalcMode;
      Size := Config.CalcSize;
      Signed := Config.CalcSigned
    end;

  with BCalcDisplay do
    begin
      CalcMode := Config.CalcMode;
      CalcSize := Config.CalcSize;
      Color := Config.DispColor;
      FontOnColor := Config.DispFontOnColor;
      FontOffColor := Config.DispFontOffColor;
      LeadingZeros := Config.LeadingZeros;
      Separator := Config.Separator;
      UseFontOff := Config.UseFontOff;
      UsePalette := Config.UsePalette
    end;

  if Config.UseWindowPos then
    begin
      Left := Config.WindowPos.X;
      Top := Config.WindowPos.Y
    end
  else
    Position := poScreenCenter;

  { Prepare the calculator and display for operation }
  UpdateDisplay;
  UpdateButtons;

  { Display the Tip of the Day dialog box, if requested.  Since this can only
    be done when the form is fully displayed, cause a timer to generate the
    required call }
  TipOfDayTimer.Enabled := Config.ShowTipOfDay
end;


{***********************************************************************}
{ The main window is about to close.  Save any user preference that has
  not already been saved }
procedure TMainWin.FormClose (Sender : TObject; var Action : TCloseAction);

begin
  with Config do
    begin
      CalcMode := Calculator.Mode;
      CalcSize := Calculator.Size;
      CalcSigned := Calculator.Signed;

      if SaveWindowPos then
        begin
          UseWindowPos := True;
          WindowPos := Point(Left, Top)
        end
    end
end;


{***********************************************************************}
{ Destroy all objects created by the main window }
procedure TMainWin.FormDestroy (Sender : TObject);

begin
  AboutWin.Free;
  OptionsWin.Free;

  Calculator.Free;
  Config.Free
end;


{***********************************************************************}
{ Handle one of the calculator buttons being pressed.  This procedure
  uses the Tag property of the button to determine what button was
  pressed; this Tag property needs to be set to the ordinal value of
  the appropriate TCalcKey value }
procedure TMainWin.CalcKeyBtnClick (Sender : TObject);

begin
  if not Calculator.HandleKey(TCalcKey((Sender as TComponent).Tag)) then
    MessageBeep(BeepType);
  UpdateDisplay
end;


{***********************************************************************}
{ Handle non-ASCII key-presses generated by the user.  Note that the
  OnKeyPress handler, FormKeyPress, handles all of the ASCII keys.
  In addition, the OnShortCut handler, FormShortCut, sends the <ENTER>
  key here; usually, navigation keys (arrows, <TAB>, <ENTER>, etc) and
  menu/action short-cuts are never sent to this procedure }
procedure TMainWin.FormKeyDown (Sender : TObject; var Key : word; Shift : TShiftState);

var
  Btn : TButton;
  Act : TAction;

begin
  { If the key-press is <SHIFT> or <CTRL> by itself, abort trying to
    handle it }
  if (Key = VK_SHIFT) or (Key = VK_CONTROL) then
    exit;

  Btn := nil;

  if Shift = [] then
    begin
      case Key of
        VK_RETURN : Btn := BtnEquals;
        VK_ESCAPE : Btn := BtnClear;
        VK_BACK   : Btn := BtnLeft
      end
    end
  else if Key = VK_INSERT then
    begin
      { Handle old-fashioned Copy and Paste short-cuts }
      Act := nil;
      if Shift = [ssCtrl] then
        Act := EditCopy
      else if Shift = [ssShift] then
        Act := EditPaste;

      if Assigned(Act) then
        begin
          Act.Execute;
          Key := 0  { Signifies that the key press was handled }
        end;

      exit
    end;

  { Handle the key press as if the button was clicked }
  if Assigned(Btn) then
    begin
      if Btn.Enabled then
        begin
          Btn.SetFocus;
          Btn.Click
        end
      else
        MessageBeep(BeepType);
      Key := 0  { Signifies that the key has been handled }
    end
end;


{***********************************************************************}
{ Process the <ENTER> key like a short-cut to the equals button.  This is
  required as the OnKeyDown handler never sees VK_RETURN by itself, which
  is where <ENTER> is actually processed }
procedure TMainWin.FormShortCut (var Msg : TWMKey; var Handled : boolean);

var
  CharCode : word;

begin
  CharCode := Msg.CharCode;
  if CharCode = VK_RETURN then
    begin
      if Assigned(OnKeyDown) then
        OnKeyDown(Self, CharCode, KeyDataToShiftState(Msg.KeyData));
      Handled := (CharCode = 0)
    end
  else
    Handled := False
end;


{***********************************************************************}
{ Convert an ASCII character into one of the buttons on the main form.
  This function is used to implement the OnKeyPress handler, as well as
  the enhanced paste function }
function TMainWin.CharToButton (C : char) : TButton;

begin
  Result := nil;
  C := UpCase(C);

  case C of
    '0'      : Result := Btn0;           { Digit keys }
    '1'      : Result := Btn1;
    '2'      : Result := Btn2;
    '3'      : Result := Btn3;
    '4'      : Result := Btn4;
    '5'      : Result := Btn5;
    '6'      : Result := Btn6;
    '7'      : Result := Btn7;
    '8'      : Result := Btn8;
    '9'      : Result := Btn9;
    'A'      : Result := BtnA;
    'B'      : Result := BtnB;
    'C'      : Result := BtnC;
    'D'      : Result := BtnD;
    'E'      : Result := BtnE;
    'F'      : Result := BtnF;

    '+'      : Result := BtnPlus;        { Operation keys }
    '-'      : Result := BtnMinus;
    '*'      : Result := BtnMultiply;
    '/'      : Result := BtnDivide;
    '%', '\' : Result := BtnMod;
    '='      : Result := BtnEquals;
    '('      : Result := BtnLParen;
    ')'      : Result := BtnRParen;

    'Q'      : Result := BtnClear;

    'M'      : Result := BtnMPlus;
    'N'      : Result := BtnMMinus;
    'I'      : Result := BtnMIn;
    'R'      : Result := BtnMR;

    'X'      : Result := BtnSwapXY;
    'Z'      : Result := BtnSwapXM;

    '!', '~' : Result := BtnNot;
    '&'      : Result := BtnAnd;
    '|'      : Result := BtnOr;
    '^'      : Result := BtnXor;
    '_'      : Result := BtnNeg;
    ':'      : Result := BtnNand;
    ';'      : Result := BtnNor;
    '#'      : Result := BtnEqv;
    '<', '[' : Result := BtnSHL;
    '>', ']' : Result := BtnSHR;
    '{'      : Result := BtnROL;
    '}'      : Result := BtnROR
  end
end;


{***********************************************************************}
{ Handle key-presses generated by the user in a keyboard-independent
  fashion.  Non-ASCII keys are handled in FormKeyDown.  This function
  simulates the user actually clicking the button }
procedure TMainWin.FormKeyPress (Sender : TObject; var Key : char);

var
  Btn : TButton;

begin
  Btn := CharToButton(Key);  { Convert Key into a button name }

  { Handle the key press as if the button was clicked }
  if Assigned(Btn) then
    begin
      if Btn.Enabled then
        begin
          Btn.SetFocus;
          Btn.Click
        end
      else
        MessageBeep(BeepType);
      Key := #0  { Signifies that the key has been handled }
    end
end;


{***********************************************************************}
{ Allow the user to set their preferences by opening the Options dialog
  box.  If the user clicked OK, change the calculator and display to
  reflect the new preferences }
procedure TMainWin.FileOptionsExecute (Sender : TObject);

var
  OldFontResID    : TResID;

begin
  { Create the dialog box if this has not already been done }
  if not Assigned(OptionsWin) then
    begin
      OptionsWin := TOptionsWin.Create(Self);
      OptionsWin.Config := Config
    end;

  OldFontResID := Config.FontResID;
  if OptionsWin.ShowModal = mrOK then
    begin
      { Set the maximum parenthesis nesting value only if it has changed.
        This is due to the fact that setting Calculator.MaxParenLevel clears
        all operations, even if the previous value was the same as the new one }
      if Calculator.MaxParenLevel <> Config.MaxParenLevel then
        Calculator.MaxParenLevel := Config.MaxParenLevel;

      { Set the new operator precedence style.  This will clear all operations
        if the new value is different from the old one }
      Calculator.OpPrecStyle := Config.OpPrecStyle;

      { Update the display with the new preferences }
      with BCalcDisplay do
        begin
          UseFontOff := Config.UseFontOff;
          Color := Config.DispColor;
          FontOnColor := Config.DispFontOnColor;
          FontOffColor := Config.DispFontOffColor;
          UsePalette := Config.UsePalette;
          LeadingZeros := Config.LeadingZeros;
          Separator := Config.Separator;
          if Config.FontResID <> OldFontResID then
            LoadFonts(Config.FontResID)
        end;

      UpdateDisplay
    end
end;


{***********************************************************************}
{ Handle the File | Exit menu item (Alt+F4) }
procedure TMainWin.FileExitExecute (Sender : TObject);

begin
  Close
end;


{***********************************************************************}
{ Copy the current value into the clipboard }
procedure TMainWin.EditCopyExecute (Sender : TObject);

begin
  Clipboard.AsText := Calculator.CurrentString
end;


{***********************************************************************}
{ Paste the clipboard contents into the calculator.  Only text can be
  pasted.  This function has two different modes of execution: normal
  and enhanced.  Normal mode pastes digits (0-F) as if they were typed;
  an illegal character aborts the operation.  Enhanced mode extends this
  to cover all keyboard short-cuts, as well as extensions for changing
  the mode, word size and signed flag }
procedure TMainWin.EditPasteExecute (Sender : TObject);

var
  S    : string;
  I, D : integer;
  C    : char;
  Btn  : TButton;

begin
  if not Clipboard.HasFormat(CF_TEXT) then
    begin
      MessageBeep(BeepType);
      exit
    end;

  D := 0;
  S := Clipboard.AsText;
  if not Config.EnhancedPaste then
    begin
      { Normal mode of operation, for backwards compatibility }
      for I := 1 to Length(S) do
        begin
          C := UpCase(S[I]);
          if C in ['0'..'9'] then
            D := Ord(C) - Ord('0')
          else if C in ['A'..'F'] then
            D := Ord(C) - Ord('A') + 10
          else
            begin
              { An illegal character was present }
              MessageBeep(BeepType);
              break
            end;

          if not Calculator.AppendDigit(D) then
            begin
              MessageBeep(BeepType);
              break
            end
        end;

      UpdateDisplay
    end
  else
    begin
      { Enhanced mode of operation }
      I := 1;
      while I <= Length(S) do
        begin
          C := S[I];
          inc(I);
          case C of
            ' ',
            #9,
            #10,
            #13,
            ',',
            '.' : { Ignore white space and punctuation } ;
            '?' : begin  { "Escape" character, to allow for commands }
                    if I <= Length(S) then
                      begin
                        C := UpCase(S[I]);
                        inc(I);
                        with Calculator do
                          case C of
                            'D'      : Mode := cmDecimal;
                            'H'      : Mode := cmHexadecimal;
                            'B'      : Mode := cmBinary;
                            'O'      : Mode := cmOctal;
                            '8'      : Size := cs8Bit;
                            '6'      : Size := cs16Bit;
                            '3'      : Size := cs32Bit;
                            '4'      : Size := cs64Bit;
                            '9'      : Size := cs96Bit;
                            '1'      : Size := cs128Bit;
                            'S'      : Signed := True;
                            'U'      : Signed := False;
                            'Q', '?' : ClearOperations
                          else
                            { Unknown command character }
                            MessageBeep(BeepType);
                            break
                          end
                      end
                    else
                      begin
                        { A trailing question mark is illegal }
                        MessageBeep(BeepType);
                        break
                      end
                  end
          else
            Btn := CharToButton(C);
            if Assigned(Btn) then
              if not Calculator.HandleKey(TCalcKey(Btn.Tag)) then
                begin
                  MessageBeep(BeepType);
                  break
                end
          end
        end;

      UpdateButtons;
      UpdateDisplay
    end
end;


{***********************************************************************}
{ Enable or disable the copy and paste actions, based on whether these
  are valid at the current time or not }
procedure TMainWin.EditUpdateActions (Sender : TObject);

var
  HasFormat : boolean;

begin
  EditCopy.Enabled := not Calculator.InError;

  HasFormat := Clipboard.HasFormat(CF_TEXT);
  if Config.EnhancedPaste then
    EditPaste.Enabled := HasFormat
  else
    EditPaste.Enabled := HasFormat and (not Calculator.InError)
end;


{***********************************************************************}
{ Select one of the calculator's bases (decimal, hexadecimal, binary or
  octal) }
procedure TMainWin.ModeBaseExecute (Sender : TObject);

begin
  if Sender = ModeDec then
    Calculator.Mode := cmDecimal
  else if Sender = ModeHex then
    Calculator.Mode := cmHexadecimal
  else if Sender = ModeBin then
    Calculator.Mode := cmBinary
  else if Sender = ModeOct then
    Calculator.Mode := cmOctal;

  UpdateButtons;
  UpdateDisplay
end;


{***********************************************************************}
{ Select one of the calculator's word size (8, 16, 32, 64, 96 or 128 bits) }
procedure TMainWin.ModeBitExecute (Sender : TObject);

begin
  if Sender = ModeBit8 then
    Calculator.Size := cs8Bit
  else if Sender = ModeBit16 then
    Calculator.Size := cs16Bit
  else if Sender = ModeBit32 then
    Calculator.Size := cs32Bit
  else if Sender = ModeBit64 then
    Calculator.Size := cs64Bit
  else if Sender = ModeBit96 then
    Calculator.Size := cs96Bit
  else if Sender = ModeBit128 then
    Calculator.Size := cs128Bit;

  UpdateButtons;
  UpdateDisplay
end;


{***********************************************************************}
{ Toggle the Signed mode action between signed and unsigned operations }
procedure TMainWin.ModeSignedExecute (Sender : TObject);

begin
  Calculator.Signed := not Calculator.Signed;

  UpdateButtons;
  UpdateDisplay
end;


{***********************************************************************}
{ Update the mode actions to be enabled or disabled, based on whether
  the calculator is in an error state }
procedure TMainWin.ModeUpdateActions (Sender : TObject);

var
  En : boolean;

begin
  En := not Calculator.InError;

  ModeDec.Enabled := En;
  ModeHex.Enabled := En;
  ModeBin.Enabled := En;
  ModeOct.Enabled := En;

  ModeBit8.Enabled := En;
  ModeBit16.Enabled := En;
  ModeBit32.Enabled := En;
  ModeBit64.Enabled := En;
  ModeBit96.Enabled := En;
  ModeBit128.Enabled := En;

  ModeSigned.Enabled := En
end;


{***********************************************************************}
{ Show the topics available in the Base Calculator help file }
procedure TMainWin.HelpTopicsExecute (Sender : TObject);

begin
  Application.HelpCommand(HELP_FINDER, 0)
end;


{***********************************************************************}
{ Display the Tip of the Day dialog box }
procedure TMainWin.HelpTipOfDayExecute (Sender : TObject);

var
  Index : integer;

begin
  { Since this procedure may have been called by TipOfDayTimer, disable
    the timer }
  TipOfDayTimer.Enabled := False;

  { Load the tips, if not already loaded }
  LoadTips;

  with TipsDialog do
    begin
      ShowOnStartup := Config.ShowTipOfDay;
      TipIndex := Config.TipOfDayIndex;

      Execute;

      Config.ShowTipOfDay := ShowOnStartup;
      Index := TipIndex + 1;
      if Index >= TipItems.Count then
        Config.TipOfDayIndex := 0
      else
        Config.TipOfDayIndex := Index
    end
end;


{***********************************************************************}
{ Use a Web browser to display the URL contained in HomePageURL.  This
  function uses ShellExecute and assumes that a Web browser has been
  correctly installed }
procedure TMainWin.HelpWebExecute (Sender : TObject);

begin
  ShellExecute(0, 'open', PChar(HomePageURL), nil, nil, SW_SHOWNORMAL)
end;


{***********************************************************************}
{ Display the About dialog box }
procedure TMainWin.HelpAboutExecute (Sender : TObject);

begin
  if not Assigned(AboutWin) then
    AboutWin := TAboutWin.Create(Self);

  AboutWin.ShowModal
end;


{***********************************************************************}
{ Display context-sensitive help related to the main display.  This
  procedure is invoked via the PopupEditMenu "What's This?" menu }
procedure TMainWin.EditWhatExecute (Sender : TObject);

var
  HC : THelpContext;

begin
  HC := (PopupEditMenu.PopupComponent as TWinControl).HelpContext;
  if HC <> 0 then
    Application.HelpContext(HC)
end;


{***********************************************************************}
{ Display context-sensitive help for the button under the "What's This?"
  menu item.  Note that this procedure is different from EditWhatExecute
  above, since the PopupComponent owner is different }
procedure TMainWin.HelpWhatExecute (Sender : TObject);

var
  Popup : TComponent;
  HC    : THelpContext;

begin
  Popup := PopupWhatMenu.PopupComponent;
  if Popup is TWinControl then
    HC := TWinControl(Popup).HelpContext
  else if Popup is TToolButton then
    HC := (TToolButton(Popup).Action as TAction).HelpContext
  else
    HC := 0;

  if HC <> 0 then
    Application.HelpCommand(HELP_CONTEXTPOPUP, HC)
end;


{***********************************************************************}
{ Check to see if an automatically-generated request for help (via an
  <F1> key-press) should be suppressed or converted into a different
  form of help.  The reason for this is that <F1> generates a HELP_CONTEXT
  command, whereas context-sensitive help requires HELP_CONTEXTPOPUP.
  Unfortunately, calling Application.HelpCommand with a parameter of
  HELP_CONTEXTPOPUP causes the help to pop up at the current mouse
  location, NOT at the position of the key.  Hence, make the user request
  help via the "What's This?" pop-up }
function TMainWin.FormHelp (Command : word; Data : integer;
                                 var CallHelp : boolean) : boolean;

begin
  CallHelp := True;

  if Command = HELP_CONTEXT then
    begin
      if Data >= HelpNoPopup then
        CallHelp := False
      else if Data >= HelpAutoPopup then
        begin
          Application.HelpCommand(HELP_CONTEXTPOPUP, Data);
          CallHelp := False
        end
    end;

  Result := True
end;


{***********************************************************************}
{ Process the WM_DISPLAYCHANGE message sent by Windows to inform applications
  of a change in the display configuration.  Since this may affect the
  palettised display used in BCalcDisplay, we should force a check }
procedure TMainWin.WMDisplayChange (var Message : TWMDisplayChange);

begin
  BCalcDisplay.DisplayChanged
end;


{***********************************************************************}
{ Load the four fonts required by BCalcDisplay from the application's
  resources.  Use FontResID as the starting font number }
procedure TMainWin.LoadFonts (FontResID : TResID);

var
  FontBM : TPicture;

begin
  FontBM := TPicture.Create;
  try
    FontBM.Bitmap.LoadFromResourceID(HInstance, FontResID + SmallFontOfs);
    BCalcDisplay.SmallFont := FontBM;
    FontBM.Bitmap.LoadFromResourceID(HInstance, FontResID + MediumFontOfs);
    BCalcDisplay.MediumFont := FontBM;
    FontBM.Bitmap.LoadFromResourceID(HInstance, FontResID + LargeFontOfs);
    BCalcDisplay.LargeFont := FontBM;
    FontBM.Bitmap.LoadFromResourceID(HInstance, FontResID + ErrorFontOfs);
    BCalcDisplay.ErrorFont := FontBM
  finally
    FontBM.Free
  end
end;


{***********************************************************************}
{ Load the tips for the Tip of the Day dialog box from TIPS.DAT.  If the
  tips have already been loaded, do not do so again }
procedure TMainWin.LoadTips;

var
  TipsFileName : string;
  Stream       : TStream;
  Size         : integer;
  Buf, S       : string;
  P, Start     : PChar;

begin
  if FTipsLoaded then
    exit;

  { Even if not loaded, do not try to load again }
  FTipsLoaded := True;

  { Use the filename supplied by the registry, with the path of the application }
  TipsFileName := Config.TipOfDayFile;
  if ExtractFilePath(TipsFileName) = '' then
    TipsFileName := ExtractFilePath(Application.ExeName) + TipsFileName;

  if not FileExists(TipsFileName) then
    exit;

  with TipsDialog.TipItems do
    begin
      BeginUpdate;
      try
        Clear;

        { Read the entire file into the buffer Buf }
        Stream := TFileStream.Create(TipsFileName, fmOpenRead or fmShareDenyWrite);
        try
          Size := Stream.Size;
          SetString(Buf, nil, Size);
          Stream.Read(Pointer(Buf)^, Size)
        finally
          Stream.Free
        end;

        { Break up the buffer in Buf into separate strings }
        P := Pointer(Buf);
        if P <> nil then
          while P^ <> #0 do
            begin
              Start := P;
              while not (P^ in [#0, #10, #13]) do
                Inc(P);
              SetString(S, Start, P - Start);

              { Don't add blank lines or lines starting with ";" }
              if (S <> '') and (S[1] <> ';') then
                Add(S);

              if P^ = #13 then
                Inc(P);
              if P^ = #10 then
                Inc(P)
            end

      finally
        EndUpdate
      end
    end
end;


{***********************************************************************}
{ Update the display to reflect the calculator state machine }
procedure TMainWin.UpdateDisplay;

begin
  with BCalcDisplay do
    begin
      CalcMode := Calculator.Mode;
      CalcSize := Calculator.Size;
      MemInUse := Calculator.MemInUse;
      CurrentCalcOp := Calculator.CurrentCalcOp;
      ConstantMode := Calculator.ConstantMode;
      ParenLevel := Calculator.ParenLevel;
      if Calculator.InError then
        DisplayMode := dmError
      else
        begin
          DisplayMode := dmNormal;
          Value := Calculator.CurrentString
        end
    end
end;


{***********************************************************************}
{ Enable or display the number keys, depending on the current mode.  In
  addition, select one of the mode, size and signed actions as checked }
procedure TMainWin.UpdateButtons;

begin
  { The brute-force method! }
  case Calculator.Mode of
    cmDecimal     : begin
                      ModeDec.Checked := True;
                      ModeHex.Checked := False;
                      ModeBin.Checked := False;
                      ModeOct.Checked := False;

                      Btn2.Enabled := True;    { Keys 0-9 are allowed }
                      Btn3.Enabled := True;
                      Btn4.Enabled := True;
                      Btn5.Enabled := True;
                      Btn6.Enabled := True;
                      Btn7.Enabled := True;
                      Btn8.Enabled := True;
                      Btn9.Enabled := True;
                      BtnA.Enabled := False;
                      BtnB.Enabled := False;
                      BtnC.Enabled := False;
                      BtnD.Enabled := False;
                      BtnE.Enabled := False;
                      BtnF.Enabled := False
                    end;
    cmHexadecimal : begin
                      ModeDec.Checked := False;
                      ModeHex.Checked := True;
                      ModeBin.Checked := False;
                      ModeOct.Checked := False;

                      Btn2.Enabled := True;    { Keys 0-F are allowed }
                      Btn3.Enabled := True;
                      Btn4.Enabled := True;
                      Btn5.Enabled := True;
                      Btn6.Enabled := True;
                      Btn7.Enabled := True;
                      Btn8.Enabled := True;
                      Btn9.Enabled := True;
                      BtnA.Enabled := True;
                      BtnB.Enabled := True;
                      BtnC.Enabled := True;
                      BtnD.Enabled := True;
                      BtnE.Enabled := True;
                      BtnF.Enabled := True
                    end;
    cmBinary      : begin
                      ModeDec.Checked := False;
                      ModeHex.Checked := False;
                      ModeBin.Checked := True;
                      ModeOct.Checked := False;

                      Btn2.Enabled := False;   { Keys 0-1 are allowed }
                      Btn3.Enabled := False;
                      Btn4.Enabled := False;
                      Btn5.Enabled := False;
                      Btn6.Enabled := False;
                      Btn7.Enabled := False;
                      Btn8.Enabled := False;
                      Btn9.Enabled := False;
                      BtnA.Enabled := False;
                      BtnB.Enabled := False;
                      BtnC.Enabled := False;
                      BtnD.Enabled := False;
                      BtnE.Enabled := False;
                      BtnF.Enabled := False
                    end;
    cmOctal       : begin
                      ModeDec.Checked := False;
                      ModeHex.Checked := False;
                      ModeBin.Checked := False;
                      ModeOct.Checked := True;

                      Btn2.Enabled := True;    { Keys 0-7 are allowed }
                      Btn3.Enabled := True;
                      Btn4.Enabled := True;
                      Btn5.Enabled := True;
                      Btn6.Enabled := True;
                      Btn7.Enabled := True;
                      Btn8.Enabled := False;
                      Btn9.Enabled := False;
                      BtnA.Enabled := False;
                      BtnB.Enabled := False;
                      BtnC.Enabled := False;
                      BtnD.Enabled := False;
                      BtnE.Enabled := False;
                      BtnF.Enabled := False
                    end;
  end;

  case Calculator.Size of
    cs8Bit   : begin
                 ModeBit8.Checked := True;
                 ModeBit16.Checked := False;
                 ModeBit32.Checked := False;
                 ModeBit64.Checked := False;
                 ModeBit96.Checked := False;
                 ModeBit128.Checked := False
               end;
    cs16Bit  : begin
                 ModeBit8.Checked := False;
                 ModeBit16.Checked := True;
                 ModeBit32.Checked := False;
                 ModeBit64.Checked := False;
                 ModeBit96.Checked := False;
                 ModeBit128.Checked := False
               end;
    cs32Bit  : begin
                 ModeBit8.Checked := False;
                 ModeBit16.Checked := False;
                 ModeBit32.Checked := True;
                 ModeBit64.Checked := False;
                 ModeBit96.Checked := False;
                 ModeBit128.Checked := False
               end;
    cs64Bit  : begin
                 ModeBit8.Checked := False;
                 ModeBit16.Checked := False;
                 ModeBit32.Checked := False;
                 ModeBit64.Checked := True;
                 ModeBit96.Checked := False;
                 ModeBit128.Checked := False
               end;
    cs96Bit  : begin
                 ModeBit8.Checked := False;
                 ModeBit16.Checked := False;
                 ModeBit32.Checked := False;
                 ModeBit64.Checked := False;
                 ModeBit96.Checked := True;
                 ModeBit128.Checked := False
               end;
    cs128Bit : begin
                 ModeBit8.Checked := False;
                 ModeBit16.Checked := False;
                 ModeBit32.Checked := False;
                 ModeBit64.Checked := False;
                 ModeBit96.Checked := False;
                 ModeBit128.Checked := True
               end;
  end;

  ModeSigned.Checked := Calculator.Signed
end;

end.

